@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
-
 import locale
+import os
 from copy import deepcopy
+from unittest import skipIf
 from xml.etree import ElementTree as etree
 import pytest
 
@@ -98,12 +98,14 @@ def locale_fixture():
 
 
 class Test_Locale:
+    @skipIf(os.name == 'nt', "Not tested on Windows")
     @pytest.mark.parametrize("loc", ["", "en_US.UTF-8", "de_DE.UTF-8"])
     def test_fromstring_numbers_locale_insensitive(self, loc, locale_fixture):
         "Case relies on that LC_ALL is set in the console."
         locale.setlocale(locale.LC_NUMERIC, loc)
         text = """<testsuites>
-        <testsuite errors="0" failures="0" hostname="hooch" name="pytest" skipped="0" tests="2" time="1000.125" timestamp="2020-02-05T10:52:33.843536">
+        <testsuite errors="0" failures="0" hostname="hooch" name="pytest" skipped="0" tests="2" time="1000.125"
+        timestamp="2020-02-05T10:52:33.843536">
         <testcase classname="test_x" file="test_x.py" line="7" name="test_comp_1" time="1,000.025"/>
         <testcase classname="test_x" file="test_x.py" line="10" name="test_comp_2" time="0.1"/>
         </testsuite>
@@ -125,6 +127,7 @@ class Test_JunitXml:
         <testcase name="testname2">
         </testcase></testsuite></testsuites>"""
         result = JUnitXml.fromstring(text)
+        assert isinstance(result, JUnitXml)
         assert len(result) == 2
         assert result.time == 0
 
@@ -133,17 +136,20 @@ class Test_JunitXml:
         <testcase name="testname1">
         </testcase></testsuite>"""
         result = JUnitXml.fromstring(text)
+        assert isinstance(result, JUnitXml)
         assert len(result) == 1
         assert result.time == 0
 
     def test_fromstring_multiple_fails(self):
         text = """<testsuites>
-        <testsuite errors="1" failures="0" hostname="hooch" name="pytest" skipped="1" tests="3" time="0.025" timestamp="2020-02-05T10:52:33.843536">
+        <testsuite errors="1" failures="0" hostname="hooch" name="pytest" skipped="1" tests="3" time="0.025"
+        timestamp="2020-02-05T10:52:33.843536">
         <testcase classname="test_x" file="test_x.py" line="7" name="test_comp_1" time="0.000"/>
         <testcase classname="test_x" file="test_x.py" line="10" name="test_comp_2" time="0.000">
         <skipped message="unconditional skip" type="pytest.skip">test_x.py:11: unconditional skip</skipped>
         <error message="test teardown failure">
-        @pytest.fixture(scope="module") def compb(): yield > raise PermissionError E PermissionError test_x.py:6: PermissionError
+        @pytest.fixture(scope="module") def compb(): yield > raise PermissionError E
+        PermissionError test_x.py:6: PermissionError
         </error>
         </testcase>
         </testsuite>
@@ -161,21 +167,27 @@ class Test_JunitXml:
 
     def test_fromroot_testsuite(self):
         text = """
-        <testsuite errors="1" failures="0" hostname="hooch" name="pytest" skipped="1" tests="3" time="0.025" timestamp="2020-02-05T10:52:33.843536">
+        <testsuite errors="1" failures="0" hostname="hooch" name="pytest" skipped="1" tests="3" time="0.025"
+        timestamp="2020-02-05T10:52:33.843536">
         <testcase classname="test_x" file="test_x.py" line="7" name="test_comp_1" time="0.000"/>
         <testcase classname="test_x" file="test_x.py" line="10" name="test_comp_2" time="0.000">
         <skipped message="unconditional skip" type="pytest.skip">test_x.py:11: unconditional skip</skipped>
         <error message="test teardown failure">
-        @pytest.fixture(scope="module") def compb(): yield > raise PermissionError E PermissionError test_x.py:6: PermissionError
+        @pytest.fixture(scope="module") def compb(): yield > raise PermissionError E
+        PermissionError test_x.py:6: PermissionError
         </error>
         </testcase>
         </testsuite>"""
         root_elemt = etree.fromstring(text)
         result = JUnitXml.fromroot(root_elemt)
-        assert isinstance(result, TestSuite)
-        assert result.errors == 1
-        assert result.skipped == 1
-        cases = list(iter(result))
+        assert isinstance(result, JUnitXml)
+        suites = list(iter(result))
+        assert len(suites) == 1
+        suite = suites[0]
+        assert isinstance(suite, TestSuite)
+        assert suite.errors == 1
+        assert suite.skipped == 1
+        cases = list(iter(suite))
         assert len(cases[0].result) == 0
         assert len(cases[1].result) == 2
         text = cases[1].result[1].text
@@ -183,18 +195,21 @@ class Test_JunitXml:
 
     def test_fromroot_testsuites(self):
         text = """<testsuites>
-        <testsuite errors="1" failures="0" hostname="hooch" name="pytest" skipped="1" tests="3" time="0.025" timestamp="2020-02-05T10:52:33.843536">
+        <testsuite errors="1" failures="0" hostname="hooch" name="pytest" skipped="1" tests="3" time="0.025"
+        timestamp="2020-02-05T10:52:33.843536">
         <testcase classname="test_x" file="test_x.py" line="7" name="test_comp_1" time="0.000"/>
         <testcase classname="test_x" file="test_x.py" line="10" name="test_comp_2" time="0.000">
         <skipped message="unconditional skip" type="pytest.skip">test_x.py:11: unconditional skip</skipped>
         <error message="test teardown failure">
-        @pytest.fixture(scope="module") def compb(): yield > raise PermissionError E PermissionError test_x.py:6: PermissionError
+        @pytest.fixture(scope="module") def compb(): yield > raise PermissionError E
+        PermissionError test_x.py:6: PermissionError
         </error>
         </testcase>
         </testsuite>
         </testsuites>"""
         root_elemt = etree.fromstring(text)
         result = JUnitXml.fromroot(root_elemt)
+        assert isinstance(result, JUnitXml)
         assert result.errors == 1
         assert result.skipped == 1
         suite = list(iter(result))[0]
@@ -685,18 +700,32 @@ class Test_TestCase:
         case.result = [Skipped()]
         assert case.is_skipped
         assert not case.is_passed
+        assert not case.is_failure
+        assert not case.is_error
 
     def test_case_is_passed(self):
         case = TestCase()
         case.result = []
         assert not case.is_skipped
         assert case.is_passed
+        assert not case.is_failure
+        assert not case.is_error
 
     def test_case_is_failed(self):
         case = TestCase()
         case.result = [Failure()]
         assert not case.is_skipped
         assert not case.is_passed
+        assert case.is_failure
+        assert not case.is_error
+
+    def test_case_is_error(self):
+        case = TestCase()
+        case.result = [Error()]
+        assert not case.is_skipped
+        assert not case.is_passed
+        assert not case.is_failure
+        assert case.is_error
 
 
 class Test_Properties:
